@@ -152,6 +152,7 @@ static void
 gda_meta_struct_init (GdaMetaStruct *mstruct) {
 	mstruct->priv = g_new0 (GdaMetaStructPrivate, 1);
 	mstruct->priv->store = NULL;
+	mstruct->priv->db_objects = NULL;
 	mstruct->priv->index = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 }
 
@@ -400,6 +401,19 @@ get_user_obj_name (const GValue *catalog, const GValue *schema, const GValue *na
 	return ret;
 }
 
+static gchar *
+prepare_sql_identifier_for_compare (gchar *str)
+{
+	if (!str || (*str == '"'))
+		return str;
+	else {
+		gchar *ptr;
+		for (ptr = str; *ptr; ptr++)
+			*ptr = g_ascii_tolower (*ptr);
+		return str;
+	}
+}
+
 /**
  * gda_meta_struct_complement
  * @mstruct: a #GdaMetaStruct object
@@ -420,12 +434,15 @@ get_user_obj_name (const GValue *catalog, const GValue *schema, const GValue *na
  * If @catalog is %NULL and @schema is not %NULL, then the database object will be the one which 
  * can be accessed by its @schema.@name name.
  *
- * Important note: @catalog, @schema and @name must respect the following convention:
+ * Important note: @catalog, @schema and @name will be used using the following convention:
  * <itemizedlist>
  *   <listitem><para>be surrounded by double quotes for a case sensitive search</para></listitem>
- *   <listitem><para>otherwise they will be converted to lower case for search</para></listitem>
+ *   <listitem><para>otherwise for case insensitive search</para></listitem>
  * </itemizedlist>
  *
+ * For more information, see the <link linkend="information_schema:sql_identifiers">
+ * meta data section about SQL identifiers</link>.
+ *
  * Returns: the #GdaMetaDbObject corresponding to the database object if no error occurred, or %NULL
  */
 GdaMetaDbObject *
@@ -453,19 +470,19 @@ gda_meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 	/* create ready to compare strings for catalog, schema and name */
 	gchar *schema_s, *name_s;
 	if (_split_identifier_string (g_value_dup_string (name), &schema_s, &name_s)) {
-		g_value_take_string ((iname = gda_value_new (G_TYPE_STRING)), gda_sql_identifier_remove_quotes (name_s));
+		g_value_take_string ((iname = gda_value_new (G_TYPE_STRING)), prepare_sql_identifier_for_compare (name_s));
 		if (schema_s)
-			g_value_take_string ((ischema = gda_value_new (G_TYPE_STRING)), gda_sql_identifier_remove_quotes (schema_s));
+			g_value_take_string ((ischema = gda_value_new (G_TYPE_STRING)), prepare_sql_identifier_for_compare (schema_s));
 	}
 	else
 		g_value_take_string ((iname = gda_value_new (G_TYPE_STRING)), 
-				     gda_sql_identifier_remove_quotes (g_value_dup_string (name)));
+				     prepare_sql_identifier_for_compare (g_value_dup_string (name)));
 	if (catalog)
 		g_value_take_string ((icatalog = gda_value_new (G_TYPE_STRING)), 
-				     gda_sql_identifier_remove_quotes (g_value_dup_string (catalog)));
+				     prepare_sql_identifier_for_compare (g_value_dup_string (catalog)));
 	if (schema && !ischema) 
 		g_value_take_string ((ischema = gda_value_new (G_TYPE_STRING)), 
-				     gda_sql_identifier_remove_quotes (g_value_dup_string (schema)));
+				     prepare_sql_identifier_for_compare (g_value_dup_string (schema)));
 
 	if (!icatalog) {
 		if (ischema) {
@@ -881,7 +898,13 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 					if (fk_nrows != ref_pk_nrows) {
 						/*gda_data_model_dump (fk_cols, stdout);
 						  gda_data_model_dump (ref_pk_cols, stdout);*/
-						fkerror = TRUE;
+						if (ref_pk_nrows > 0)
+							fkerror = TRUE;
+						else {
+							/* not an error, only the referenced table is not present
+							 * in the meta store, which is possible if the meta
+							 * store was only partially updated */
+						}
 					}
 					else {
 						gint n;
@@ -1015,7 +1038,7 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 
  onerror:
 	if (dbo)
-		gda_meta_db_object_free (dbo);
+		dbo->obj_type = GDA_META_DB_UNKNOWN;
 
 	return NULL;
 }
@@ -1077,6 +1100,8 @@ array_type_to_sql (GdaMetaStore *store, const GValue *specific_name)
  * If @catalog is %NULL, then any catalog will be used, and
  * if @schema is %NULL then any schema will be used (if @schema is %NULL then catalog must also be %NULL).
  *
+ * Please refer to gda_meta_struct_complement() form more information.
+ *
  * Returns: TRUE if no error occurred
  */
 gboolean
@@ -1094,7 +1119,7 @@ gda_meta_struct_complement_schema (GdaMetaStruct *mstruct, const GValue *catalog
 		"ORDER BY table_schema, table_name";
 	const gchar *sql2 = "SELECT table_short_name, table_full_name, table_owner, table_name "
 		"FROM _tables WHERE table_catalog = ##cat::string AND table_schema = ##schema::string "
-		"AND table_type='VIEW' "
+		"AND table_type LIKE '%VIEW%' "
 		"ORDER BY table_schema, table_name";
 	
 	/* schema is known, catalog unknown */
@@ -1102,7 +1127,7 @@ gda_meta_struct_complement_schema (GdaMetaStruct *mstruct, const GValue *catalog
 		"FROM _tables WHERE table_schema = ##schema::string AND table_type LIKE '%TABLE%' "
 		"ORDER BY table_schema, table_name";
 	const gchar *sql4 = "SELECT table_short_name, table_full_name, table_owner, table_name, table_catalog, table_schema "
-		"FROM _tables WHERE table_schema = ##schema::string AND table_type='VIEW' "
+		"FROM _tables WHERE table_schema = ##schema::string AND table_type LIKE '%VIEW%' "
 		"ORDER BY table_schema, table_name";
 
 	/* schema and catalog are unknown */
@@ -1110,7 +1135,7 @@ gda_meta_struct_complement_schema (GdaMetaStruct *mstruct, const GValue *catalog
 		"FROM _tables WHERE table_type LIKE '%TABLE%' "
 		"ORDER BY table_schema, table_name";
 	const gchar *sql6 = "SELECT table_short_name, table_full_name, table_owner, table_name, table_catalog, table_schema "
-		"FROM _tables WHERE table_type='VIEW' "
+		"FROM _tables WHERE table_type LIKE '%VIEW%' "
 		"ORDER BY table_schema, table_name";
 
 	g_return_val_if_fail (GDA_IS_META_STRUCT (mstruct), FALSE);
@@ -1207,12 +1232,18 @@ real_gda_meta_struct_complement_all (GdaMetaStruct *mstruct, gboolean default_on
 	const gchar *sql2 = "SELECT table_catalog, table_schema, table_name, table_short_name, table_full_name, table_owner "
 		"FROM _tables WHERE table_short_name = table_name AND table_type='VIEW' "
 		"ORDER BY table_schema, table_name";
+	const gchar *sql3 = "SELECT table_catalog, table_schema, table_name, table_short_name, table_full_name, table_owner "
+		"FROM _tables WHERE table_type LIKE '%TABLE%' "
+		"ORDER BY table_schema, table_name";
+	const gchar *sql4 = "SELECT table_catalog, table_schema, table_name, table_short_name, table_full_name, table_owner "
+		"FROM _tables WHERE table_type='VIEW' "
+		"ORDER BY table_schema, table_name";
 
 	g_return_val_if_fail (GDA_IS_META_STRUCT (mstruct), FALSE);
 	g_return_val_if_fail (mstruct->priv->store, FALSE);
 
 	/* tables */
-	model = gda_meta_store_extract (mstruct->priv->store, sql1, error, NULL);
+	model = gda_meta_store_extract (mstruct->priv->store, default_only ? sql1 : sql3, error, NULL);
 	if (!model)
 		return FALSE;
 	nrows = gda_data_model_get_n_rows (model);
@@ -1235,7 +1266,7 @@ real_gda_meta_struct_complement_all (GdaMetaStruct *mstruct, gboolean default_on
 	g_object_unref (model);
 
 	/* views */
-	model = gda_meta_store_extract (mstruct->priv->store, sql2, error, NULL);
+	model = gda_meta_store_extract (mstruct->priv->store, default_only ? sql2 : sql4, error, NULL);
 	if (!model)
 		return FALSE;
 	nrows = gda_data_model_get_n_rows (model);
@@ -1266,9 +1297,11 @@ real_gda_meta_struct_complement_all (GdaMetaStruct *mstruct, gboolean default_on
  *
  * This method is similar to gda_meta_struct_complement() and gda_meta_struct_complement_all()
  * but creates #GdaMetaDbObject for all the
- * database object which are useable using only their short name (that is which do not need to be prefixed by 
+ * database object which are usable using only their short name (that is which do not need to be prefixed by 
  * the schema in which they are to be used).
  *
+ * Please refer to gda_meta_struct_complement() form more information.
+ *
  * Returns: TRUE if no error occurred
  */
 gboolean
@@ -1284,6 +1317,8 @@ gda_meta_struct_complement_default (GdaMetaStruct *mstruct, GError **error)
  *
  * This method is similar to gda_meta_struct_complement() and gda_meta_struct_complement_default()
  * but creates #GdaMetaDbObject for all the database object.
+ * 
+ * Please refer to gda_meta_struct_complement() form more information.
  *
  * Returns: TRUE if no error occurred
  */
@@ -1302,6 +1337,8 @@ gda_meta_struct_complement_all (GdaMetaStruct *mstruct, GError **error)
  * This method is similar to gda_meta_struct_complement() but creates #GdaMetaDbObject for all the dependencies
  * of @dbo.
  *
+ * Please refer to gda_meta_struct_complement() form more information.
+ *
  * Returns: TRUE if no error occurred
  */
 gboolean
@@ -1380,11 +1417,24 @@ build_pass (GSList *objects, GSList *ordered_list)
 static gint
 db_object_sort_func (GdaMetaDbObject *dbo1, GdaMetaDbObject *dbo2)
 {
-	gint retval;
-	retval = strcmp (dbo1->obj_schema, dbo2->obj_schema);
-	if (retval)
-		return retval;
-	return strcmp (dbo1->obj_name, dbo2->obj_name);
+	gint retval = 0;
+	if (dbo1->obj_schema && dbo2->obj_schema) {
+		retval = strcmp (dbo1->obj_schema, dbo2->obj_schema);
+		if (retval)
+			return retval;
+	}
+	else if (dbo1->obj_schema)
+		return 1;
+	else if (dbo2->obj_schema)
+		return -1;
+
+	if (dbo1->obj_name && dbo2->obj_name)
+		return strcmp (dbo1->obj_name, dbo2->obj_name);
+	else if (dbo1->obj_name)
+		return 1;
+	else if (dbo2->obj_name)
+		return -1;
+	return 0;
 }
 
 /**
@@ -1497,7 +1547,8 @@ _meta_struct_get_db_object (GdaMetaStruct *mstruct, const GValue *catalog, const
  * Get a list of all the #GdaMetaDbObject structures representing database objects in @mstruct. Note that
  * no #GdaMetaDbObject structure must not be modified.
  *
- * Returns: a new #GSList list of pointers which must be destroyed after usage using g_slist_free().
+ * Returns: a new #GSList list of pointers to #GdaMetaDbObject structures which must be destroyed after
+ * usage using g_slist_free(). The individual #GdaMetaDbObject must not be modified.
  */
 GSList *
 gda_meta_struct_get_all_db_objects (GdaMetaStruct *mstruct)
@@ -1537,13 +1588,13 @@ gda_meta_struct_get_db_object (GdaMetaStruct *mstruct, const GValue *catalog, co
 	g_return_val_if_fail (!schema || (G_VALUE_TYPE (schema) == G_TYPE_STRING), NULL);
 
 	/* prepare identifiers */
-	g_value_take_string ((iname = gda_value_new (G_TYPE_STRING)), gda_sql_identifier_remove_quotes (g_value_dup_string (name)));
+	g_value_take_string ((iname = gda_value_new (G_TYPE_STRING)), prepare_sql_identifier_for_compare (g_value_dup_string (name)));
 	if (catalog)
 		g_value_take_string ((icatalog = gda_value_new (G_TYPE_STRING)), 
-				     gda_sql_identifier_remove_quotes (g_value_dup_string (catalog)));
+				     prepare_sql_identifier_for_compare (g_value_dup_string (catalog)));
 	if (schema) 
 		g_value_take_string ((ischema = gda_value_new (G_TYPE_STRING)), 
-				     gda_sql_identifier_remove_quotes (g_value_dup_string (schema)));
+				     prepare_sql_identifier_for_compare (g_value_dup_string (schema)));
 
 	dbo = _meta_struct_get_db_object (mstruct, icatalog, ischema, iname);
 	if (icatalog) gda_value_free (icatalog);
@@ -1737,6 +1788,7 @@ gda_meta_db_object_free_contents (GdaMetaDbObject *dbo)
 		TO_IMPLEMENT;
 	}
 	g_slist_free (dbo->depend_list);
+	memset (dbo, 0, sizeof (GdaMetaDbObject));
 }
 
 static void
@@ -2049,8 +2101,8 @@ determine_db_object_from_short_name (GdaMetaStruct *mstruct,
 				g_free (obj_schema);
 				return FALSE;
 			}
-			g_value_take_string ((sv = gda_value_new (G_TYPE_STRING)), gda_sql_identifier_remove_quotes (obj_schema));
-			g_value_take_string ((nv = gda_value_new (G_TYPE_STRING)), gda_sql_identifier_remove_quotes (obj_name));
+			g_value_take_string ((sv = gda_value_new (G_TYPE_STRING)), prepare_sql_identifier_for_compare (obj_schema));
+			g_value_take_string ((nv = gda_value_new (G_TYPE_STRING)), prepare_sql_identifier_for_compare (obj_name));
 			retval = determine_db_object_from_schema_and_name (mstruct, in_out_type, out_catalog, 
 									   out_short_name, out_full_name, out_owner, 
 									   sv, nv);
@@ -2185,7 +2237,7 @@ copyerror:
  * @error: a place to store errors, or %NULL
  *
  * Adds @dbo to the database objects known to @mstruct. In any case (whether an error occured or not)
- * @dbo's responsability is then transferred to @smtruct and should
+ * @dbo's responsibility is then transferred to @smtruct and should
  * not be used after calling this function (it may have been destroyed). If you need a pointer to the #GdaMetaDbObject
  * for a database object, use gda_meta_struct_get_db_object().
  *
