@@ -294,10 +294,15 @@ struct _GdaServerProviderClass {
 	/* distributed transaction */
 	GdaServerProviderXa       *xa_funcs; /* it is a pointer! => set to %NULL if unsupported by provider */
 
+	/* SQL identifiers quoting */
+	gchar                  *(* identifier_quote)    (GdaServerProvider *provider, GdaConnection *cnc,
+							 const gchar *id,
+							 gboolean for_meta_store, gboolean force_quotes);
+
+	/* Async. handling@ */
 	gboolean                (*handle_async)          (GdaServerProvider *provider, GdaConnection *cnc, GError **error);
 
 	/* Padding for future expansion */
-	void                    (*_gda_reserved2)        (void);
 	void                    (*_gda_reserved3)        (void);
 	void                    (*_gda_reserved4)        (void);
 	void                    (*_gda_reserved5)        (void);
